/*
 PureMVC standard Javascript port by Frederic Saunier <frederic.saunier@puremvc.org>
 PureMVC - Copyright(c) 2006-2011 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @classDescription
 * A base singleton <code>Facade</code> implementation.
 * 
 * <P>In PureMVC, the <code>Facade</code> class assumes these responsibilities:
 * 
 * <UL>
 * 
 * <LI>Initializing the <code>Model</code>, <code>View</code>
 * and <code>Controller</code> singletons.
 * <LI>Providing all the applicable methods of the <code>Model</code>,
 * <code>View</code>, & <code>Controller</code> singletons.
 * <LI>Providing a single point of contact to the application for
 * registering <code>Command</code>s and notifying <code>Observer</code>s.
 * 
 * <P>This <code>Facade</code> implementation is a singleton
 * and cannot be instantiated directly, but instead calls the static singleton
 * factory method <code>Facade.getInstance()</code>.
 * 
 * @see org.puremvc.js.standard.core.Controller Controller
 * @see org.puremvc.js.standard.core.Model Model
 * @see org.puremvc.js.standard.core.View View
 * @see org.puremvc.js.standard.patterns.observer.Notification Notification
 * @see org.puremvc.js.standard.patterns.mediator.Mediator Mediator
 * @see org.puremvc.js.standard.patterns.proxy.Proxy Proxy
 * @see org.puremvc.js.standard.patterns.command.SimpleCommand SimpleCommand
 * @see org.puremvc.js.standard.patterns.command.MacroCommand MacroCommand
 *
 * @constructor
 */
function Facade(){}

/**
 * Initialize a <code>Facade</code> instance.
 * 
 * @return {Facade}
 * 		This <code>Facade</code> instance.
 */
Facade.prototype.initialize = function()
{
	this.initializeFacade();
	
	return this;
}

/**
 * The <code>View</code> singleton.
 *
 * @type {View}
 * @private
 */
Facade.prototype.view = null;

/**
 * The <code>Model</code> singleton.
 *
 * @type {Model}
 * @private
 */
Facade.prototype.model = null;

/**
 * The <code>Controller</code> singleton.
 *
 * @type {Controller}
 * @private
 */
Facade.prototype.controller = null;

/**
 * @private
 * Called automatically by the constructor.
 * init the Singleton <code>Facade</code> instance.
 *
 * <P>
 * Override in your subclass to do any subclass specific
 * initializations. Be sure to extend the <code>Facade</code> with the
 * methods and properties on your implementation and call
 * <code>Facade.initializeFacade()</code>.
 */
Facade.prototype.initializeFacade = function()
{
	this.initializeModel();
	this.initializeController();
	this.initializeView();
}

/**
 * @private
 * init the <code>Model</code>.
 *
 * <P>
 * Called by the <code>initializeFacade</code> method. Override this
 * method in your subclass of <code>Facade</code> if one or both of the
 * following are true:
 * 
 * <UL>
 * <LI>You wish to init a different <code>IModel</code>.
 * <LI>You have <code>Proxy</code>s to register with the
 * <code>Model</code> that do not retrieve a reference to the
 * <code>Facade</code> at construction time.
 *
 * <P>
 * Note: This method is <i>rarely</i> overridden; in practice you are
 * more likely to use a <code>Command</code> to create and register
 * <code>Proxy</code>s with the <code>Model</code>, since
 * <code>Proxy</code>s with mutable data will likely need to send 
 * <code>Notification</code>s and thus will likely want to fetch a
 * reference to the <code>Facade</code> during their construction.
 */
Facade.prototype.initializeModel = function()
{
	this.model = Model.getInstance();
}

/**
 * @private
 * init the <code>Controller</code>.
 *
 * <P>
 * Called by the <code>initializeFacade</code> method.
 * Override this method in JSON Object <code>Facade</code>
 * definition if one or both of the following are true:
 * 
 * <UL>
 * <LI>You wish to init a different <code>Controller</code>.
 * <LI>You have <code>Command</code>s to register with the
 * <code>Controller</code> at startup.</code>.
 */
Facade.prototype.initializeController = function()
{
	this.controller = Controller.getInstance();
}

/**
 * @private
 * init the <code>View</code>.
 *
 * <P>
 * Called by the <code>initializeFacade</code> method.
 * Override this method in your subclass of <code>Facade</code>
 * if one or both of the following are true:
 * 
 * <UL>
 * <LI>You wish to init a different <code>IView</code>.</LI>
 * <LI>You have <code>Observer</code>s to register with the
 * <code>View</code></LI>
 * </UL>
 * 
 * <P>
 * Note: This method is <i>rarely</i> overridden; in practice you are
 * more likely to use a <code>Command</code> to create and register
 * <code>Mediator</code>s with the <code>View</code>, since
 * <code>IMediator</code> instances will need to send
 * <code>INotification</code>s and thus will likely want to fetch a
 * reference to the <code>Facade</code> during their construction.
 */
Facade.prototype.initializeView = function()
{
	this.view = View.getInstance();
}

/**
 * Register a <code>Command</code> with the <code>Controller</code> by
 * <code>Notification</code> name.
 *
 * @param {String} notificationName
 * 		The name of the <code>Notification</code> to associate the
 * 		<code>Command</code> with.
 *
 * @param {Class} commandClassRef
 * 		A reference to the Class of the <code>Command</code>.
 */
Facade.prototype.registerCommand = function( notificationName/*String*/, commandClassRef/*Class*/ )
{
	this.controller.registerCommand( notificationName, commandClassRef );
}

/**
 * Remove a previously registered <code>Command</code> to
 * <code>Notification</code> mapping from the <code>Controller</code>.
 *
 * @param {String} notificationName
 * 		The name of the <code>Notification</code> to remove the
 * 		<code>Command</code> mapping for.
 */
Facade.prototype.removeCommand = function(notificationName/*String*/ )
{
	this.controller.removeCommand(notificationName);
}

/**
 * Check if a <code>Command</code> is registered for a given
 * <code>Notification</code>.
 *
 * @param {String} notificationName
 * 		The name of the <code>Notification</code> to verify for the
 * 		existence of a <code>Command</code> mapping for.
 *
 * @return {Boolean}
 * 		A <code>Command</code> is currently registered for the given
 * 		<i>notificationName</i>.
 */
Facade.prototype.hasCommand = function( notificationName/*String*/ )
{
	return this.controller.hasCommand(notificationName);
},

/**
 * Register a <code>Proxy</code> with the <code>Model</code> by name.
 *
 * @param proxy {Proxy}
 * 		The <code>Proxy</code> instance to be registered with the
 * 		<code>Model</code>.
 */
Facade.prototype.registerProxy = function( proxy/*Proxy*/ )
{
	this.model.registerProxy(proxy);
}

/**
 * Retrieve a <code>Proxy</code> from the <code>Model</code> by name.
 *
 * @param {String} proxyName
 * 		The name of the <code>Proxy</code> to be retrieved.
 *
 * @return {Proxy}
 * 		The <code>Proxy</code> instance previously registered with the
 * 		given <i>proxyName</i>.
 */
Facade.prototype.retrieveProxy = function( proxyName/*String*/ )
{
	return this.model.retrieveProxy(proxyName);
}

/**
 * Remove an <code>Proxy</code> from the <code>Model</code> by name.
 *
 * @param {String} proxyName
 * 		The <code>Proxy</code> to remove from the <code>Model</code>.
 *
 * @return {Proxy}
 * 		The <code>Proxy</code> that was removed from the
 * 		<code>Model</code>.
 */
Facade.prototype.removeProxy = function(proxyName /*String*/)
{
	return this.model.removeProxy(proxyName);
}

/**
 * Check if a <code>Proxy</code> is registered.
 *
 * @param {String} proxyName
 * 		The <code>Proxy</code> to verify the existence of a 
 * 		registration with the <code>Model</code>.
 *
 * @return {Boolean}
 * 		A <code>Proxy</code> is currently registered with the given
 * 		<i>proxyName</i>.
 */
Facade.prototype.hasProxy = function(proxyName /*String*/)
{
	return this.model.hasProxy(proxyName);
}

/**
 * Register a <code>Mediator</code> with the <code>View</code>.
 *
 * @param {Mediator} mediator
 * 		A reference to the <code>Mediator</code>.
 */
Facade.prototype.registerMediator = function( mediator/*Mediator*/ )
{
	this.view.registerMediator(mediator);
}

/**
 * Retrieve an <code>IMediator</code> from the <code>View</code>.
 *
 * @param {String} mediatorName
 * 		The name of the registered <code>Mediator</code> to retrieve.
 *
 * @return {Mediator}
 * 		The <code>Mediator</code> previously registered with the given
 * 		<i>mediatorName</i>.
 */
Facade.prototype.retrieveMediator = function( mediatorName/*String*/ )
{
	return this.view.retrieveMediator(mediatorName);
}

/**
 * Remove an <code>Mediator</code> from the <code>View</code>.
 *
 * @param {String} mediatorName
 * 		The name of the <code>Mediator</code> to be removed.
 *
 * @return {Mediator}
 * 		The <code>Mediator</code> that was removed from the 
 * 		<code>View</code>.
 */
Facade.prototype.removeMediator = function( mediatorName/*String*/ )
{
	return this.view.removeMediator(mediatorName);
}

/**
 * Check if a <code>Mediator</code> is registered or not.
 *
 * @param {String} mediatorName
 * 		The name of the <code>Mediator</code> to verify the existence
 * 		of a registration for.
 *
 * @return {Boolean}
 * 		A <code>Mediator</code> is registered with the given
 * 		<i>mediatorName</i>.
 */
Facade.prototype.hasMediator = function( mediatorName/*String*/ )
{
	return this.view.hasMediator(mediatorName);
}

/**
 * Create and send a <code>Notification</code>.
 *
 * <P>Keeps us from having to construct new notification instances in
 * our implementation code.
 *
 * @param {String} notificationName
 * 		The name of the notification to send.
 *
 * @param {Object} body
 * 		The body of the notification to send.
 *
 * @param {String} type
 * 		The type of the notification to send.
 */
Facade.prototype.sendNotification = function( notificationName/*String*/, body/*Object*/, type/*String*/ )
{
	this.notifyObservers( new Notification().initialize( notificationName, body, type ) );
}

/**
 * Notify <code>Observer</code>s.
 *
 * <P>
 * This method is left <strong>public</strong> mostly for backward
 * compatibility, and to allow you to send custom notification classes
 * using the <code>Facade</code>.
 *
 *<P>
 * Usually you should just call <i>sendNotification</i> and pass the
 * parameters, never having to construct the <code>Notification</code>
 * yourself.
 *
 * @param {Notification} notification
 * 		The <code>Notification</code> to have the <code>View</code>
 * 		notify <code>Observers</code> of.
 */
Facade.prototype.notifyObservers = function( notification )
{
	this.view.notifyObservers(notification);
}

/**
 * @memberOf Facade
 * @type {Facade}
 * @private
 */
Facade.instance = null;

/**
 * Retrieve the singleton instance of the <code>Facade</code>.
 * 
 * @memberOf Facade
 */
Facade.getInstance = function()
{
	if( !Facade.instance )
		Facade.instance = new Facade().initialize();

	return Facade.instance;
}
/*
 PureMVC standard Javascript port by Frederic Saunier <frederic.saunier@puremvc.org>
 PureMVC - Copyright(c) 2006-2011 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @classDescription
 * <P>A base <code>Notification</code> implementation.</P>
 * <P>
 * PureMVC does not rely upon underlying event models such
 * as the one provided with Flash, and ActionScript 3 does
 * not have an inherent event model.</P>
 *
 * <P>
 * The Observer pattern as implemented within PureMVC exists
 * to support event-driven communication between the
 * application and the actors of the MVC triad (Model, View
 * and Controller.</P>
 *
 * <P>
 * Notifications are not meant to be a replacement for Events
 * in Flex/Flash/AIR/Javascript. Generally, <code>Mediator</code>
 * implementors place event listeners on their view components, which they
 * then handle in the usual way. This may lead to the broadcast of <code>Notification</code>s to
 * trigger <code>Command</code>s or to communicate with other <code>Mediators</code>.
 * <code>IProxy</code> and <code>Command</code>
 * instances communicate with each other and <code>Mediator</code>s
 * by broadcasting <code>Notification</code>s.</P>
 *
 * <P>
 * A key difference between Flash <code>Event</code>s and PureMVC
 * <code>Notification</code>s is that <code>Event</code>s follow the
 * 'Chain of Responsibility' pattern, 'bubbling' up the display hierarchy
 * until some parent component handles the <code>Event</code>, while
 * PureMVC <code>Notification</code>s follow a 'Publish/Subscribe'
 * pattern. PureMVC classes need not be related to each other in a
 * parent/child relationship in order to communicate with one another
 * using <code>Notification</code>s.
 *
 * @see org.puremvc.js.standard.patterns.observer.Observer Observer
 * 
 * @constructor
 */
function Notification(){}

/**
 * Initialize a <code>Notification</code> instance.
 *
 * @param {String} name
 * 		The name of the notification.
 *
 * @param {Object} body
 * 		(optional) Body data to send with the notification.
 * 
 * @param {String} type (optional)
 * 		Type identifier of the notification.
 * 
 * @return {Notification}
 * 		This <code>Notification</code> instance.
 */
Notification.prototype.initialize = function
(
	name /*String*/,
	body/*Object*/,
	type/*String*/
)
{			
	this.name = name;
	this.body = body;
	this.type = type;
	
	return this;
}

/**
 * The name of the notification.
 * 
 * @type {String}
 * @private
 */
Notification.prototype.name = null;

/**
 * The body data to send with the notification.
 * 
 * @type {Object} 
 * @private
 */
Notification.prototype.body = null;

/**
 * The type identifier of the notification.
 * 
 * @type {String} 
 * @private
 */
Notification.prototype.type = null;

/**
 * Get the name of the <code>Notification</code> instance.
 *
 * @return {String}
 * 		The name of the <code>Notification</code> instance.
 */
Notification.prototype.getName = function()/*String*/
{
	return this.name;
}

/**
 * Set the body of the <code>Notification</code> instance.
 *
 * @param {Object} body
 * 		The body of the notification.
 */
Notification.prototype.setBody = function( body/*Object*/ )
{
	this.body = body;
}

/**
 * Get the body of the <code>Notification</code> instance.
 *
 * @return {Object}
 * 		The body for the notification.
 */
Notification.prototype.getBody = function()/*Object*/
{
	return this.body;
}

/**
 * Set the type of the <code>Notification</code> instance.
 *
 * @param {String} type
 * 		The type identifier for the notification.
 */
Notification.prototype.setType = function( type/*String*/ )
{
	this.type = type;
}

/**
 * Get the type of the <code>Notification</code> instance.
 *
 * @return {String}
 * 		The type identifier for the notification.
 */
Notification.prototype.getType = function()
{
	return this.type;
}

/**
 * Get a textual representation of the <code>Notification</code> instance.
 *
 * @return {String}
 * 		The textual representation of the <code>Notification</code>
 * 		instance.
 */
Notification.prototype.toString = function()/*String*/
{
	var msg/*String*/ = "Notification Name: " + this.getName();
	msg += "\nBody:" + (( this.getBody() == null ) ? "null" : this.getBody().toString());
	msg += "\nType:" + (( this.getType() == null ) ? "null" : this.getType());

	return msg;
}
/*
 PureMVC standard Javascript port by Frederic Saunier <frederic.saunier@puremvc.org>
 PureMVC - Copyright(c) 2006-2011 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @classDescription
 * The Base <code>Notifier</code> implementation.
 *
 * <P>
 * <code>MacroCommand</code>, <code>Command</code>,
 * <code>Mediator</code> and <code>Proxy</code> all have a need to send
 * <code>Notifications</code>.
 * 
 * <P>
 * The <code>Notifier</code> base class provides a common method
 * called <code>sendNotification</code> that relieves implementation
 * code of the necessity to actually construct
 * <code>Notification</code>s.
 *
 * <P>
 * The <code>Notifier</code> class, which all of the above mentioned
 * classes extend, provides an initd reference to the
 * <code>Facade</code> singleton, which is required by the convenience
 * method <code>sendNotification</code>	for sending
 * <code>Notifications</code>, but it also eases implementation as
 * these classes have frequent <code>Facade</code> interactions and
 * uusually require access to the facade anyway.
 * 
 * @see org.puremvc.js.standard.patterns.facade.Facade Facade
 * 
 * @constructor
 */
function Notifier(){}

/**
 * Initialize a <code>Notifier</code> instance.
 * 
 * @return {Notifier}
 * 		This <code>Notifier</code> instance.
 */
Notifier.prototype.initialize = function()
{
	this.facade = Facade.getInstance();
	
	return this;
}

/**
 * Facade of the <code>Notifier</code> object.
 * 
 * @private
 */
Notifier.prototype.facade = null;
	
/**
 * Create and send a <code>Notification</code>.
 *
 * <P>
 * Keeps us from having to construct new Notification
 * instances in our implementation code.
 * 
 * @param {String} notificationName
 * 		The name of the notification to send.
 * 
 * @param {Object} body
 * 		The (optional) body of the notification.
 *
 * @param {String} type
 * 		The (optional) type of the notification.
 */
Notifier.prototype.sendNotification = function( notificationName/*String*/, body/*Object */, type/*String*/ )
{
	this.facade.sendNotification( notificationName, body, type );
}
/*
 PureMVC standard Javascript port by Frederic Saunier <frederic.saunier@puremvc.org>
 PureMVC - Copyright(c) 2006-2011 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @classDescription
 * A base <code>Observer</code> implementation.
 *
 * <P>
 * An <code>Observer</code> is an object that encapsulates information
 * about an interested object with a method that should
 * be called when a particular <code>Notification</code> is broadcast.
 *
 * <P>
 * In PureMVC, the <code>Observer</code> class assumes these responsibilities:
 * <UL>
 * <LI>Encapsulate the notification (callback) method of the interested object.
 * <LI>Encapsulate the notification context (this) of the interested object.
 * <LI>Provide methods for setting the notification method and context.
 * <LI>Provide a method for notifying the interested object.
 *
 * @see org.puremvc.js.standard.core.View View
 * @see org.puremvc.js.standard.patterns.observer.Notification Notification
 * 
 * @constructor
 */
function Observer(){}

/**
 * Initialize an <code>Observer</code> instance.
 *
 * @param {Function} notifyMethod
 * 		The notification method of the interested object.
 * 
 * @param {Object} notifyContext
 * 		The notification context of the interested object.
 * 
 * @return {Observer}
 * 		This <code>Observer</code> instance.
 */
Observer.prototype.initialize = function( notifyMethod/*Function*/, notifyContext/*Object*/ )
{
	this.setNotifyMethod( notifyMethod );
	this.setNotifyContext( notifyContext );

	return this;
}

/**
 * The notification method of the interested object.
 * 
 * @type {Function}
 * @private
 */
Observer.prototype.notify = null;

/**
 * The notification context of the interested object.
 * 
 * @type {Object}
 * @private
 */
Observer.prototype.context = null;

/**
 * Get the notification method.
 *
 * @return {Function}
 * 		The notification (callback) method of the interested object.
 */
Observer.prototype.getNotifyMethod = function()
{
	return this.notify;
}

/**
 * Set the notification method.
 *
 * <P>The notification method should take one parameter of type
 * <code>Notification</code>.</P>
 *
 * @param {Function} notifyMethod
 * 		The notification (callback) method of the interested object.
 */
Observer.prototype.setNotifyMethod = function( notifyMethod/*Function*/ )
{
	this.notify = notifyMethod;
}

/**
 * Get the notification context.
 *
 * @return {Object}
 * 		The notification context (<code>this</code>) of the interested
 * 		object.
 */
Observer.prototype.getNotifyContext = function()
{
	return this.context;
}

/**
 * Set the notification context.
 *
 * @param {Object} notifyContext
 * 		The notification context (this) of the interested object.
 */
Observer.prototype.setNotifyContext = function( notifyContext/*Object */ )
{
	this.context = notifyContext;
}

/**
 * Notify the interested object.
 *
 * @param {Notification} notification
 * 		The <code>Notification</code> to pass to the interested object's
 * 		notification method.
 */
Observer.prototype.notifyObserver = function( notification )
{
	this.getNotifyMethod().call( this.getNotifyContext(), notification );
}

/**
 * Compare an object to the notification context.
 *
 * @param {Object} object
 * 		The object to compare.
 *
 * @return {Boolean}
 * 		The object and the notification context are the same.
 */
Observer.prototype.compareNotifyContext = function( object/*Object*/ )
{
	return object === this.getNotifyContext();
}
/*
 PureMVC standard Javascript port by Frederic Saunier <frederic.saunier@puremvc.org>
 PureMVC - Copyright(c) 2006-2011 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @classDescription
 * The <code>Controller</code> class for PureMVC.
 *
 * <P>
 * A singleton <code>Controller</code> implementation.
 *
 * <P>
 * In PureMVC, the <code>Controller</code> class follows the
 * 'Command and Controller' strategy, and assumes these responsibilities:
 * 
 * <UL>
 * <LI>Remembering which <code>SimpleCommand</code>s or
 * <code>MacroCommand</code>s are intended to handle which
 * <code>Notification</code>s.
 * <LI>Registering itself as an <code>Observer</code> with the
 * <code>View</code> for each <code>Notification</code> that it has a
 * <code>SimpleCommand</code> or <code>MacroCommand</code> mapping for.
 * <LI>Creating a new instance of the proper <code>SimpleCommand</code>
 * or <code>MacroCommand</code> to handle a given <code>Notification</code>
 * when notified by the <code>View</code>.
 * <LI>Calling the <code>SimpleCommand</code>'s
 * or <code>MacroCommand</code>'s  <code>execute</code>
 * method, passing in the <code>Notification</code>.
 *
 * <P>
 * Your application must register <code>ICommand</code>s with the
 * <code>Controller</code>.
 *
 * <P>
 * The simplest way is to subclass </code>Facade</code>,
 * and use its <code>initializeController</code> method to add your
 * registrations.
 * 
 * @see org.puremvc.js.standard.core.View View
 * @see org.puremvc.js.standard.patterns.observer.Observer Observer
 * @see org.puremvc.js.standard.patterns.observer.Notification Notification
 * @see org.puremvc.js.standard.patterns.command.SimpleCommand SimpleCommand
 * @see org.puremvc.js.standard.patterns.command.MacroCommand MacroCommand
 * 
 * @constructor
 */
function Controller(){}

/**
 * Initialize a <code>Controller</code> instance.
 * 
 * @return {Controller}
 * 		This <code>Controller</code> instance. 
 */
Controller.prototype.initialize = function()
{
	this.commandMap = {};
	this.initializeController();
	
	return this;
}

/**
 * The <code>View</code> singleton.
 *
 * @type {View}
 * @private
 */
Controller.prototype.view = null;

/**
 * Mapping of <code>Notification<code> names to
 * <code>Command</code> class references.
 *
 * @type {Object}
 * @private
 */
Controller.prototype.commandMap = null;

/**
 * Called automatically by the constructor.
 * Retains a reference to the <code>View</code> singleton.
 * 
 * @private
 */
Controller.prototype.initializeController = function()
{
	this.view = View.getInstance();
}

/**
 * If a <code>SimpleCommand</code> or <code>MacroCommand</code> has
 * previously been registered to handle the given
 * <code>Notification</code>, then it is executed.
 *
 * @param {Notification} note
 * 		A <code>Notification</code>.
 */
Controller.prototype.executeCommand = function( note )
{
	var commandClassRef/*Notification*/ = this.commandMap[note.getName()];
	if( commandClassRef )
	{
		var command/*SimpleCommand*/ = new commandClassRef().initialize();
		command.execute(note);
	}
}

/**
 * Register a particular <code>SimpleCommand</code> or
 * <code>MacroCommand</code> class as the handler
 * for a particular <code>Notification</code>.
 *
 * <P>
 * If a <code>Command</code> has already been registered to
 * handle <code>Notification</code>s with this name, it is no longer
 * used, the new <code>Command</code> is used instead.</P>
 *
 * The <code>Observer</code> for the new <code>Command</command> is
 * only created if this is the first time a <code>Command</code> has
 * been regisered for this <code>Notification</code> name.
 *
 * @param {String} notificationName
 * 		The name of the <code>Notification</code>.
 *
 * @param {Function} commandClassRef
 * 		The <code>Class</code> of the <code>Command</code>.
 */
Controller.prototype.registerCommand = function( notificationName/*String*/, commandClassRef/*Function*/ )
{
	if( !this.commandMap[notificationName] ) 
	{
		var observer/*Observer*/ = new Observer().initialize( this.executeCommand, this );
		this.view.registerObserver( notificationName, observer );
	};

	this.commandMap[notificationName] = commandClassRef;
}

/**
 * Check if a <code>Command</code> is registered for a given
 * <code>Notification</code>.
 *
 * @param {String} notificationName
 * 		The name of the <code>Notification</code> to verify the
 * 		existence of its registration.
 * 
 * @return {Boolean}
 * 		A <code>Command</code> is currently registered for the given
 * 		<code>notificationName</code>.
 */
Controller.prototype.hasCommand = function( notificationName/*String*/ )
{
	return this.commandMap[notificationName] ? true : false;
}

/**
 * Remove a previously registered <code>SimpleCommand</code>
 * or <code>MacroCommand</code> to <code>Notification</code> mapping.
 *
 * @param {String} notificationName
 * 		The name of the <code>Notification</code> to remove the
 * 		<code>SimpleCommand</code> or <code>MacroCommand</code>	mapping
 * 		for.
 */
Controller.prototype.removeCommand = function( notificationName/*String*/ )
{
	if( this.hasCommand(notificationName) )
	{
		this.view.removeObserver(notificationName, this);
		delete this.commandMap[notificationName];
	}
}

/**
 * @memberOf Controller
 * @type {Controller}
 * @private
 */
Controller.instance = null;

/**
 * Retrieve the singleton instance of the <code>Controller</code>.
 *
 * @memberOf Controller
 * @return {Controller}
 */
Controller.getInstance = function()
{
	if( !Controller.instance ) 
		Controller.instance = new Controller().initialize();

	return Controller.instance;
}
/*
 PureMVC standard Javascript port by Frederic Saunier <frederic.saunier@puremvc.org>
 PureMVC - Copyright(c) 2006-2011 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @classDescription
 * The <code>Model</code> class for PureMVC.
 *
 * <P>
 * In PureMVC, the <code>Model</code> class provides
 * access to model objects (Proxies) by named lookup.
 *
 * <P>
 * A singleton <code>Model</code> implementation.
 *
 * <P>
 * The <code>Model</code> assumes these responsibilities:
 *
 * <UL>
 * <LI>Maintain a cache of <code>Proxy</code> instances.
 * <LI>Provide methods for registering, retrieving, and removing
 * <code>Proxy</code> instances.
 *
 * <P>Your application must register <code>Proxy</code> instances with the
 * <code>Model</code>. Typically, you use a <code>SimpleCommand</code> to
 * create and register <code>Proxy</code> instances once the
 * <code>Facade</code> has initd the core actors.
 * 
 * @see org.puremvc.js.standard.patterns.proxy.Proxy Proxy
 * 
 * @constructor
 */
function Model(){}

/**
 * Initialize a <code>Model</code> instance.
 * 
 * @return {Model}
 * 		This <code>Model</code> instance.
 */
Model.prototype.initialize = function()
{
	this.proxyMap = {};
	this.initializeModel();
	
	return this;
}

/**
 * HashTable of <code>Proxy</code> instances
 * registered with the <code>Model</code>
 * 
 * @type {Object}
 * @private
 */
Model.prototype.proxyMap = null; 

/**
 * Register a <code>Proxy</code> with the <code>Model</code>.
 *
 * @param {Proxy} proxy
 *		A <code>Proxy</code> to be held by the <code>Model</code>.
 */
Model.prototype.registerProxy = function( proxy/*Proxy*/ )
{
	var name/*String*/ = proxy.getProxyName();
	if( typeof name == "undefined" )
		return;

	this.proxyMap[name] = proxy;
	proxy.onRegister();
},

/**
 * Retrieve an <code>IProxy</code> from the <code>Model</code>.
 *
 * @param {String} proxyName
 *		The name of the <code>Proxy</code> to retrieve.
 *
 * @return {Proxy}
 *		The <code>Proxy</code> instance previously registered with the
 *		given <code>proxyName</code> or an explicit <code>null</code>
 *		if it doesn't exists.
 */
Model.prototype.retrieveProxy = function( proxyName/*String*/ )
{
	var proxy/*Proxy*/ = this.proxyMap[proxyName];
	
	//Here we don't want to return "undefined"
	return proxy ? proxy : null;
},

/**
 * Check if a <code>Proxy</code> is registered.
 *
 * @param {String} proxyName
 *		The name of the <code>Proxy</code> to verify the existence of
 		its registration.
 *
 * @return {Boolean}
 *		A Proxy is currently registered with the given
 		<code>proxyName</code>.
 */
Model.prototype.hasProxy = function( proxyName/*String*/ )
{
	return this.proxyMap[proxyName] ? true : false;
},

/**
 * Remove a <code>Proxy</code> from the <code>Model</code>.
 *
 * @param {String} proxyName
 *		The name of the <code>Proxy</code> instance to be removed.
 *
 * @return {Proxy}
 *		The <code>Proxy</code> that was removed from the
 *		<code>Model</code> or an explicit <code>null</null> if the
 *		<code>Proxy</code> didn't exist.
 */
Model.prototype.removeProxy = function( proxyName/*String*/ )
{
	var proxy/*Proxy*/ = this.proxyMap[proxyName];
	if( !proxy )
		return null;
		
	delete this.proxyMap[proxyName];
	proxy.onRemove();
	return proxy;
},

/**
 * init the singleton <code>Model</code> instance.
 *
 * <P>
 * Called automatically by the constructor. This
 * is the opportunity to init the singleton
 * instance in a subclass without overriding the
 * constructor.
 */
Model.prototype.initializeModel = function(){}

/**
 * @type {Model}
 * @private
 */
Model.instance = null;

/**
 * Retrieve the singleton instance of the <code>Model</code>.
 *
 * @return {Model}
 * 		The singleton instance of the <code>Model</code>.
 */
Model.getInstance = function()
{
	if( !Model.instance ) 
		Model.instance = new Model().initialize();

	return Model.instance;
}
/*
 PureMVC standard Javascript port by Frederic Saunier <frederic.saunier@puremvc.org>
 PureMVC - Copyright(c) 2006-2011 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @classDescription
 * The <code>View</code> class in PureMVC.
 * 
 * <P>
 * A singleton <code>View</code> implementation. In PureMVC, the
 * <code>View</code> class assumes these responsibilities:
 * 
 * <UL>
 * <LI>Maintain a cache of <code>Mediator</code> instances.
 * <LI>Provide methods for registering, retrieving, and removing
 * <code>IMediator</code>s.
 * <LI>Notifiying <code>Mediator</code>s when they are registered or removed.
 * <LI>Managing the <code>Observer</code> lists for each
 * <code>Notification</code> in the application.
 * <LI>Providing a method for attaching <code>Observer</code>s to an
 * <code>INotification</code>'s <code>Observer</code> list.
 * <LI>Providing a method for broadcasting a <code>Notification</code>.
 * <LI>Notifying the <code>Observer</code>s of a given
 * <code>Notification</code> when it broadcasts.
 *
 * @see org.puremvc.js.standard.patterns.mediator.Mediator Mediator
 * @see org.puremvc.js.standard.patterns.observer.Observer Observer
 * @see org.puremvc.js.standard.patterns.observer.Notification Notification
 * 
 * @constructor
 */
function View(){}

/**
 * Initialize a <code>View</code> instance.
 * 
 * @return {View}
 * 		This <code>View</code> instance.
 */
View.prototype.initialize = function()
{
	this.mediatorMap = {};
	this.observerMap = {};
	this.initializeView();
	
	return this;
}

/**
 * Mapping of <code>Mediator</code> names to <code>Mediator</code>
 * instances.
 *
 * @type {Object}
 * 
 * @private
 */
View.prototype.mediatorMap = null;

/**
 * Mapping of <code>Notification</code> names to
 * <code>Observers</code> lists.
 *
 * @type {Object}
 * 
 * @private
 */
View.prototype.observerMap = null;

/**
 * Init the singleton <code>View</code> instance.
 *
 * <P>
 * Called automatically by the constructor. This
 * is the opportunity to init the singleton
 * instance in a subclass without overriding the
 * constructor.
 * 
 * @private
 */
View.prototype.initializeView = function(){}

/**
 * Register an <code>Observer</code> to be notified of
 * <code>Notifications</code> with a given name.
 *
 * @param {String} notificationName
 * The name of the <code>Notification</code>s to notify this
 * <code>Observer</code> of.
 * 
 * @param {Observer} observer
 * 		The <code>Observer</code> to register.
 */
View.prototype.registerObserver = function( notificationName/*String*/, observer/*Observer*/ )
{
	var observers/*Array*/ = this.observerMap[notificationName];
	if( observers )
		observers.push(observer);
	else
		this.observerMap[notificationName] = [observer];
}

/**
 * Notify the <code>Observer</code>s for a particular
 * <code>Notification</code>.
 *
 * <P>
 * All previously attached <code>Observer</code>s for this
 * <code>Notification</code>'s list are notified and are passed a reference
 * to the <code>Notification</code> in the order in which they were
 * registered.
 *
 * @param {Notification} notification
 * 		The <code>Notification</code> to notify <code>Observer</code>s
 * 		of.
 */
View.prototype.notifyObservers = function( notification )
{
	var name/*String*/ = notification.getName();

	if( this.observerMap[name] )
	{
		// Copy the array.
		var observers/*Array*/ = this.observerMap[name].slice(0);
		var len/*Number*/ = observers.length;
		for( var i=0; i<len; i++ )
		{
			var observer/*Observer*/ = observers[i];
			observer.notifyObserver(notification);
		}
	}
}

/**
 * Remove the <code>Observer</code> for a given <i>notifyContext</i>
 * from an <code>Observer</code> list for a given
 * <code>Notification</code> name.
 *
 * @param {String} notificationName
 * 		Which <code>Observer</code> list to remove from.
 *
 * @param {Object} notifyContext
 * 		Remove the <code>Observer</code> with this object as its
 *		<i>notifyContext</i>.
 */
View.prototype.removeObserver = function( notificationName/*String*/, notifyContext/* Object */)
{
	var observers/*Array*/ = this.observerMap[notificationName];
	var i/*Number*/ = observers.length;

	while( i-- )
	{
		var observer/*Observer*/ = observers[i];
		if( observer.compareNotifyContext(notifyContext) )
		{
			observers.splice( i, 1 );
			break;
		}
	}

	// Remove empty observer lists.
	if( !observers.length )
		delete this.observerMap[notificationName];
}

/**
 * Register an <code>IMediator</code> instance with the <code>View</code>.
 *
 * <P>
 * Registers the <code>IMediator</code> so that it can be retrieved by name,
 * and further interrogates the <code>IMediator</code> for its
 * <code>INotification</code> interests.
 *
 * <P>
 * If the <code>IMediator</code> returns any <code>INotification</code>
 * names to be notified about, an <code>Observer</code> is created to
 * encapsulate the <code>IMediator</code> instance's
 * <code>handleNotification</code> method and register it as an
 * <code>Observer</code> for all <code>INotification</code>s the
 * <code>IMediator</code> is interested in.
 *
 * @param {Mediator} mediator
 * 		A reference to the <code>Mediator</code> instance.
 */
View.prototype.registerMediator = function( mediator/*Mediator*/ )
{
	var name/*String*/ = mediator.getMediatorName();
	
	// Do not allow re-registration (you must removeMediator first)
	if( this.mediatorMap[name] )
		return;

	// Register the Mediator for retrieval by name
	this.mediatorMap[name] = mediator;
	
	// Register Mediator as an observer for each of its notification interests
	var interests/*Array*/ = mediator.listNotificationInterests();
	var len/*Number*/ = interests.length;
	if( len )
	{
    	// Register Mediator as Observer for its list of Notification interests
		var observer/*Observer*/ = new Observer().initialize( mediator.handleNotification, mediator );
		for( var i=0; i<len; i++ )
			this.registerObserver( interests[i], observer );
	}

	mediator.onRegister();
}

/**
 * Retrieve a <code>Mediator</code> from the <code>View</code>.
 *
 * @param {String} mediatorName
 *		The name of the <code>IMediator</code> instance to retrieve.
 *
 * @return {Mediator}
 *		The <code>Mediator</code> instance previously registered with
 *		the given <i>mediatorName</i> or an explicit <code>null</code>
 *		if it doesn't exists.
 */
View.prototype.retrieveMediator = function( mediatorName/*String*/ )
{
	var mediator/*Mediator*/ = this.mediatorMap[mediatorName];

	//Here we don't want to return "undefined"
	return mediator ? mediator : null;
}

/**
 * Check if a <code>Mediator</code> is registered or not.
 *
 * @param {String} mediatorName
 *		Name of the <code>IMediator</code> instance to verify the
 *		existence of its registration.
 *
 * @return {Boolean}
 *		A <code>Mediator</code> is registered with the given
 *		<i>mediatorName</i>.
 */
View.prototype.hasMediator = function( mediatorName/*String*/ )
{
	return this.mediatorMap[mediatorName] ? true : false;
}

/**
 * Remove a <code>Mediator</code> from the <code>View</code>.
 *
 * @param {String} mediatorName
 *		Name of the <code>IMediator</code> instance to be removed.
 *
 * @return {Mediator}
 *		The <code>Mediator</code> that was removed from the
 		<code>View</code> or an explicit <code>null</null> if the
 		<code>Mediator</code> didn't exist.
 */
View.prototype.removeMediator = function( mediatorName/*String*/ )
{
	var mediator/*Mediator*/ = this.mediatorMap[mediatorName];
	if( !mediator )
		return null;

	var interests/*Array*/ = mediator.listNotificationInterests();
	var i/*Number*/ = interests.length;
	while (i--) 
		this.removeObserver( interests[i], mediator );
	
	delete this.mediatorMap[mediatorName];
	mediator.onRemove();
	return mediator;	
}

/**
 * @type {View}
 * @private
 */
View.instance = null;

/**
 * Retrieve the singleton instance of the <code>View</code>.
 * @memberof Puremvc.core.View
 */
View.getInstance = function()
{
	if( !View.instance ) 
		View.instance = new View().initialize();

	return View.instance;
}
/*
 PureMVC standard Javascript port by Frederic Saunier <frederic.saunier@puremvc.org>
 PureMVC - Copyright(c) 2006-2011 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @classDescription <P>A base <code>Command</code> implementation that executes other <code>Command</code>s.
 *
 * <P>
 * A <code>MacroCommand</code> maintains an list of
 * <code>ICommand</code> Class references called <i>SubCommands</i>.
 *
 * <P>
 * When <code>execute</code> is called, the <code>MacroCommand</code>
 * instantiates and calls <code>execute</code> on each of its <i>SubCommands</i> turn.
 * Each <i>SubCommand</i> will be passed a reference to the original
 * <code>INotification</code> that was passed to the <code>MacroCommand</code>'s
 * <code>execute</code> method.
 *
 * <P>
 * Unlike <code>SimpleCommand</code>, your subclass
 * should not override <code>execute</code>, but instead, should
 * override the <code>initMacroCommand</code> method,
 * calling <code>addSubCommand</code> once for each <i>SubCommand</i>
 * to be executed.
 * 
 * @extends org.puremvc.js.standard.patterns.observer.Notifier Notifier
 * 
 * @constructor
 */
function MacroCommand(){}

/**
 * @extends org.puremvc.js.standard.patterns.observer.Notifier Notifier
 */
MacroCommand.prototype = new Notifier();

/**
 * @override
 *
 * Initialize a <code>MacroCommand</code> instance.
 * 
 * @return {MacroCommand}
 * 		This <code>MacroCommand</code> instance.
 */
MacroCommand.prototype.initialize = function()
{
	Notifier.prototype.initialize.call(this);

	this.subCommands = [];
	this.initializeMacroCommand();
	
	return this;
}

/**
 * An array of <code>SimpleCommands</code>
 * or subclasses of.
 * @type {Array}
 * @private
 */
MacroCommand.prototype.subCommands = null;

/**
 * Init the <code>MacroCommand</code>.
 *
 * <P>
 * In your subclass, override this method to
 * init the <code>MacroCommand</code>'s <i>subCommands</i>
 * list with <code>ICommand</code> class references like
 * this:
 *
 * <pre>
 *    // init MyMacroCommand
 *    initMacroCommand= function()
 *    {
 *    	this.addSubCommand(FirstCommand);
 *      this.addSubCommand(SecondCommand);
 *      this.addSubCommand(ThirdCommand);
 *    }
 * </pre>
 *
 * <P>Note that <i>subCommands</i> may be any <code>Command</code>
 * implementor <code>MacroCommand</code>s or
 * <code>SimpleCommand</code>s are both acceptable.
 */
MacroCommand.prototype.initializeMacroCommand = function(){}

/**
 * Add an entry to <i>subCommands</i> list.
 *
 * <P>
 * The <i>subCommands</i> will be called in First In/First Out (FIFO)
 * order.
 *
 * @param {Function} commandClassRef
 * 		A reference to the <code>Class</code> of the <code>ICommand</code>.
 */
MacroCommand.prototype.addSubCommand = function( commandClassRef )
{
	this.subCommands.push( commandClassRef );
}

/**
 * Execute this <code>MacroCommand</code>'s <i>SubCommands</i>.
 *
 * <P>
 * The <i>subCommands</i> will be called in First In/First Out (FIFO)
 * order.
 *
 * @param {Notification} notification
 * 		The <code>Notification</code> object to be passed to each entry
 * 		of <i>subCommands</i> list.
 * 
 * @final
 */
MacroCommand.prototype.execute = function( notification )
{
	var len/*Number*/ = this.subCommands.length;
	for( var i/*Number*/=0; i<len; i++ )
	{
		var commandClassRef = this.subCommands[i];
		var commandInstance/*Command*/ = new commandClassRef().initialize();
		commandInstance.execute(notification);
	}
}
/*
 PureMVC standard Javascript port by Frederic Saunier <frederic.saunier@puremvc.org>
 PureMVC - Copyright(c) 2006-2011 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @classDescription
 * A base <code>Command</code> implementation.
 * 
 * <P>
 * Your subclass should override the <code>execute</code> method where
 * your business logic will handle the <code>Notification</code>.
 *
 * @extends org.puremvc.js.standard.patterns.observer.Notifier Notifier
 */
function SimpleCommand(){}

/**
 * @extends org.puremvc.js.standard.patterns.observer.Notifier Notifier
 */
SimpleCommand.prototype = new Notifier();

/**
 * @override
 *
 * Initialize a <code>SimpleCommand</code> instance.
 * 
 * @return {SimpleCommand}
 * 		This <code>SimpleCommand</code> instance.
 */
SimpleCommand.prototype.initialize = function()
{
	Notifier.prototype.initialize.call(this);

	return this;
}

/**
* Fulfill the use-case initiated by the given <code>Notification</code>.
*
* <P>
* In the Command Pattern, an application use-case typically begins with
* some user action, which results in a <code>Notification</code> being
* broadcast, which is handled by business logic in the
* <code>execute</code> method of an <code>ICommand</code>.
*
* @param {Notification} notification 
* 		The <code>Notification</code> to handle.
*/
SimpleCommand.prototype.execute= function( notification ){}
/*
 PureMVC standard Javascript port by Frederic Saunier <frederic.saunier@puremvc.org>
 PureMVC - Copyright(c) 2006-2011 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @classDescription
 * <P>A base <code>Mediator</code> implementation.
 * 
 * <P>
 * Typically, a <code>Mediator</code> will be written to serve
 * one specific control or group controls and so,
 * will not have a need to be dynamically named.
 * 
 * @see org.puremvc.js.standard.patterns.observer.Notification Notification
 * @extends org.puremvc.js.standard.patterns.observer.Notifier Notifier
 * 
 * @constructor
 */
function Mediator(){}

/**
 * @extends org.puremvc.js.standard.patterns.observer.Notifier Notifier
 */
Mediator.prototype = new Notifier();

/**
 * @override
 *
 * Initialize a <code>Mediator</code> instance.
 *
 * @param {String} mediatorName
 * 		The name of the <code>Mediator</code>.
 *
 * @param {Object} viewComponent
 * 		The <code>Mediator</code>'s view component.
 * 
 * @return {Mediator}
 * 		This <code>Mediator</code> instance.
 */
Mediator.prototype.initialize = function( mediatorName/*String*/, viewComponent/*Object*/ )
{
	Notifier.prototype.initialize.call(this);

	this.mediatorName = (mediatorName != null) ? mediatorName : Mediator.NAME;
	this.viewComponent = viewComponent;
	
	return this;
}

/**
 * The name of the <code>Mediator</code>.
 * 
 * @type {String}
 * @private
 */
Mediator.prototype.mediatorName = null;

/**
 * The <code>Mediator</code>'s view component.
 * 
 * @type {Object}
 * @private
 */
Mediator.prototype.viewComponent = null;


/**
 * List the <code>Notification</code> names this
 * <code>Mediator</code> is interested in being notified of.
 *
 * @return {Array}
 * 		The list of notifications names in which is interested the
 * 		<code>Mediator</code>.
 */
Mediator.prototype.listNotificationInterests = function()
{
	return [];
}

/**
 * Get the name of the <code>Mediator</code>.
 *
 * @return {String}
 * 		The <code>Mediator</code> name.
 */
Mediator.prototype.getMediatorName = function()
{
	return this.mediatorName;
}

/**
 * Get the <code>Mediator</code>'s view component.
 *
 * @return {Object}
 * 		The view component.
 */
Mediator.prototype.getViewComponent = function()
{
	return this.viewComponent;
}

/**
 * Set the <code>Mediator</code>'s view component.
 *
 * @param {Object} viewComponent
 * 		The view component.
 */
Mediator.prototype.setViewComponent = function( viewComponent/*Object*/ )
{
	this.viewComponent = viewComponent;
}

/**
 * Handle <code>INotification</code>s.
 *
 * <P>
 * Typically this will be handled in a switch statement,
 * with one 'case' entry per <code>Notification</code>
 * the <code>Mediator</code> is interested in.
 *
 * @param {Notification} notification
 * 		The notification instance to be handled.
 */
Mediator.prototype.handleNotification = function( notification ){}

/**
 * Called by the View when the Mediator is registered.
 * This method is usually overridden as needed by the subclass.
 */
Mediator.prototype.onRegister = function(){}

/**
 * Called by the View when the Mediator is removed.
 * This method is usually overridden as needed by the subclass.
 */
Mediator.prototype.onRemove = function(){}

/**
 * Default name of the <code>Mediator</code>.
 * 
 * @type {String}
 * @memberOf Mediator
 * @constant
 */
Mediator.NAME = "Mediator";
/*
 PureMVC standard Javascript port by Frederic Saunier <frederic.saunier@puremvc.org>
 PureMVC - Copyright(c) 2006-2011 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @classDescription
 * A base <code>Proxy</code> implementation.
 * 
 * <P>In PureMVC, <code>Proxy</code> classes are used to manage parts of the
 * application's data model.
 *
 * <P>A <code>Proxy</code> might simply manage a reference to a local data object,
 * in which case interacting with it might involve setting and
 * getting of its data in synchronous fashion.
 *
 * <P><code>Proxy</code> classes are also used to encapsulate the application's
 * interaction with remote services to store or retrieve data, in which case,
 * we adopt an asynchronous idiom; setting data (or calling a method) on the
 * <code>Proxy</code> and listening for a <code>Notification</code> to be sent
 * when the <code>Proxy</code> has retrieved the data from the service.
 *
 * @see org.puremvc.js.standard.core.Model Model
 *
 * @extends org.puremvc.js.standard.patterns.observer.Notifier Notifier
 * 
 * @constructor
 */
function Proxy(){}

/**
 * @extends org.puremvc.js.standard.patterns.observer.Notifier Notifier
 */
Proxy.prototype = new Notifier();

/**
 * @override
 *
 * Initialize a <code>Proxy</code> instance.
 *
 * @param {String} proxyName
 * 		The name of the <code>Proxy</code>.
 *
 * @param {Object} data
 * 		An initial data object to be held by the <code>Proxy</code>.
 * 
 * @return {Proxy}
 * 		This <code>Proxy</code> instance.
 */
Proxy.prototype.initialize = function( proxyName/*String*/, data/*Object*/ )
{
	Notifier.prototype.initialize.call(this);

	this.proxyName = (proxyName != null) ? proxyName : Proxy.NAME;
	this.data = data;
	
	return this;
}

/**
 * The data object controlled by the <code>Proxy</code>.
 *
 * @type {Object}
 * @private
 */
Proxy.prototype.data = null;

/**
 * The name of the <code>Proxy</code>.
 * 
 * @type {String}
 * @private
 */
Proxy.prototype.proxyName = null;

/**
 * Gets the proxyName.
 *
 * @return {String}
 * 		The name of the proxy.
 */
Proxy.prototype.getProxyName = function()
{
	return this.proxyName;
}

/**
 * Sets the data object.
 *
 * @param {Object} data
 * 		The data to set.
 */
Proxy.prototype.setData = function( data/*Object*/ )
{
	this.data = data;
}

/**
 * Gets the data.
 *
 * @return {Object}
 * 		The data held in the <code>Proxy.
 */
Proxy.prototype.getData = function()
{
	return this.data;
}

/**
 * Called by the Model when the <code>Proxy</code> is registered.
 * This method is usually overridden as needed by the subclass.
 */
Proxy.prototype.onRegister = function(){}

/**
 * Called by the Model when the <code>Proxy</code> is removed.
 * This method is usually overridden as needed by the subclass.
 */
Proxy.prototype.onRemove = function(){}

/**
 * The default name of the <code>Proxy</code>
 * 
 * @type {String}
 * @memberOf {Proxy}
 * @constant
 */
Proxy.NAME = "Proxy";
