/*
 PureMVC standard Javascript port by Frederic Saunier <frederic.saunier@puremvc.org>
 PureMVC - Copyright(c) 2006-2011 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @classDescription
 * The <code>Controller</code> class for PureMVC.
 *
 * <P>
 * A singleton <code>Controller</code> implementation.
 *
 * <P>
 * In PureMVC, the <code>Controller</code> class follows the
 * 'Command and Controller' strategy, and assumes these responsibilities:
 * 
 * <UL>
 * <LI>Remembering which <code>SimpleCommand</code>s or
 * <code>MacroCommand</code>s are intended to handle which
 * <code>Notification</code>s.
 * <LI>Registering itself as an <code>Observer</code> with the
 * <code>View</code> for each <code>Notification</code> that it has a
 * <code>SimpleCommand</code> or <code>MacroCommand</code> mapping for.
 * <LI>Creating a new instance of the proper <code>SimpleCommand</code>
 * or <code>MacroCommand</code> to handle a given <code>Notification</code>
 * when notified by the <code>View</code>.
 * <LI>Calling the <code>SimpleCommand</code>'s
 * or <code>MacroCommand</code>'s  <code>execute</code>
 * method, passing in the <code>Notification</code>.
 *
 * <P>
 * Your application must register <code>ICommand</code>s with the
 * <code>Controller</code>.
 *
 * <P>
 * The simplest way is to subclass </code>Facade</code>,
 * and use its <code>initializeController</code> method to add your
 * registrations.
 * 
 * @see org.puremvc.js.standard.core.View View
 * @see org.puremvc.js.standard.patterns.observer.Observer Observer
 * @see org.puremvc.js.standard.patterns.observer.Notification Notification
 * @see org.puremvc.js.standard.patterns.command.SimpleCommand SimpleCommand
 * @see org.puremvc.js.standard.patterns.command.MacroCommand MacroCommand
 * 
 * @constructor
 */
function Controller(){}

/**
 * Initialize a <code>Controller</code> instance.
 * 
 * @throws {Error}
 * 		Throws an error if an instance for this singleton has already
 * 		been constructed.
 * 
 * @return {Controller}
 * 		This <code>Controller</code> instance. 
 */
Controller.prototype.initialize = function()
{
	if( Controller.instance )
		throw Error( View.SINGLETON_MSG );

	this.commandMap = {};
	this.initializeController();
	
	return this;
}

/**
 * The <code>View</code> singleton.
 *
 * @type {View}
 * @private
 */
Controller.prototype.view = null;

/**
 * Mapping of <code>Notification<code> names to
 * <code>Command</code> class references.
 *
 * @type {Object}
 * @private
 */
Controller.prototype.commandMap = null;

/**
 * Called automatically by the constructor.
 * Retains a reference to the <code>View</code> singleton.
 * 
 * @private
 */
Controller.prototype.initializeController = function()
{
	this.commandMap = {};
	this.view = View.getInstance();
}

/**
 * If a <code>SimpleCommand</code> or <code>MacroCommand</code> has
 * previously been registered to handle the given
 * <code>Notification</code>, then it is executed.
 *
 * @param {Notification} note
 * 		A <code>Notification</code>.
 */
Controller.prototype.executeCommand = function( note )
{
	var commandClassRef/*Notification*/ = this.commandMap[note.getName()];
	if( commandClassRef )
	{
		var command/*SimpleCommand*/ = new commandClassRef().initialize();
		command.execute(note);
	}
}

/**
 * Register a particular <code>SimpleCommand</code> or
 * <code>MacroCommand</code> class as the handler
 * for a particular <code>Notification</code>.
 *
 * <P>
 * If a <code>Command</code> has already been registered to
 * handle <code>Notification</code>s with this name, it is no longer
 * used, the new <code>Command</code> is used instead.</P>
 *
 * The <code>Observer</code> for the new <code>Command</command> is
 * only created if this is the first time a <code>Command</code> has
 * been regisered for this <code>Notification</code> name.
 *
 * @param {String} notificationName
 * 		The name of the <code>Notification</code>.
 *
 * @param {Function} commandClassRef
 * 		The <code>Class</code> of the <code>Command</code>.
 */
Controller.prototype.registerCommand = function( notificationName/*String*/, commandClassRef/*Function*/ )
{
	if( !this.commandMap[notificationName] ) 
	{
		var observer/*Observer*/ = new Observer().initialize( this.executeCommand, this );
		this.view.registerObserver( notificationName, observer );
	};

	this.commandMap[notificationName] = commandClassRef;
}

/**
 * Check if a <code>Command</code> is registered for a given
 * <code>Notification</code>.
 *
 * @param {String} notificationName
 * 		The name of the <code>Notification</code> to verify the
 * 		existence of its registration.
 * 
 * @return {Boolean}
 * 		A <code>Command</code> is currently registered for the given
 * 		<code>notificationName</code>.
 */
Controller.prototype.hasCommand = function( notificationName/*String*/ )
{
	return this.commandMap[notificationName] ? true : false;
}

/**
 * Remove a previously registered <code>SimpleCommand</code>
 * or <code>MacroCommand</code> to <code>Notification</code> mapping.
 *
 * @param {String} notificationName
 * 		The name of the <code>Notification</code> to remove the
 * 		<code>SimpleCommand</code> or <code>MacroCommand</code>	mapping
 * 		for.
 */
Controller.prototype.removeCommand = function( notificationName/*String*/ )
{
	if( this.hasCommand(notificationName) )
	{
		this.view.removeObserver(notificationName, this);
		delete this.commandMap[notificationName];
	}
}

/**
 * @constant
 * @type {String}
 * @private
 */
Controller.SINGLETON_MSG = "Controller singleton already constructed!";

/**
 * @type {Controller}
 * @private
 */
Controller.instance = null;

/**
 * Retrieve the singleton instance of the <code>Controller</code>.
 *
 * @return {Controller}
 * 		The singleton instance of the <code>Controller</code>.
 */
Controller.getInstance = function()
{
	if( !Controller.instance ) 
		Controller.instance = new Controller().initialize();

	return Controller.instance;
}