/*
 PureMVC standard Javascript port by Frederic Saunier <frederic.saunier@puremvc.org>
 PureMVC - Copyright(c) 2006-2011 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @classDescription <P>A base <code>Command</code> implementation that executes other <code>Command</code>s.
 *
 * <P>
 * A <code>MacroCommand</code> maintains an list of
 * <code>ICommand</code> Class references called <i>SubCommands</i>.
 *
 * <P>
 * When <code>execute</code> is called, the <code>MacroCommand</code>
 * instantiates and calls <code>execute</code> on each of its <i>SubCommands</i> turn.
 * Each <i>SubCommand</i> will be passed a reference to the original
 * <code>INotification</code> that was passed to the <code>MacroCommand</code>'s
 * <code>execute</code> method.
 *
 * <P>
 * Unlike <code>SimpleCommand</code>, your subclass
 * should not override <code>execute</code>, but instead, should
 * override the <code>initMacroCommand</code> method,
 * calling <code>addSubCommand</code> once for each <i>SubCommand</i>
 * to be executed.
 * 
 * @extends org.puremvc.js.standard.patterns.observer.Notifier Notifier
 * 
 * @constructor
 */
function MacroCommand(){}

/**
 * @extends org.puremvc.js.standard.patterns.observer.Notifier Notifier
 */
MacroCommand.prototype = new Notifier();

/**
 * @override
 *
 * Initialize a <code>MacroCommand</code> instance.
 * 
 * @return {MacroCommand}
 * 		This <code>MacroCommand</code> instance.
 */
MacroCommand.prototype.initialize = function()
{
	Notifier.prototype.initialize.call(this);

	this.subCommands = [];
	this.initializeMacroCommand();
	
	return this;
}

/**
 * An array of <code>SimpleCommands</code>
 * or subclasses of.
 * @type {Array}
 * @private
 */
MacroCommand.prototype.subCommands = null;

/**
 * Initialize the <code>MacroCommand</code>.
 *
 * <P>
 * In your subclass, override this method to
 * initialize the <code>MacroCommand</code>'s <i>subCommands</i>
 * list with <code>ICommand</code> class references like
 * this:
 *
 * <pre>
 *    // Initialize MyMacroCommand
 *    initializeMacroCommand: function()
 *    {
 *    	this.addSubCommand(FirstCommand);
 *      this.addSubCommand(SecondCommand);
 *      this.addSubCommand(ThirdCommand);
 *    }
 * </pre>
 *
 * <P>
 * Note that <i>subCommands</i> may be any <code>Command</code>
 * implementor.
 *
 * <P>
 * In the JavaScript version it means that it only needs to
 * implement an execute method and inherits from Notifier.
 */
MacroCommand.prototype.initializeMacroCommand = function(){}

/**
 * Add an entry to <i>subCommands</i> list.
 *
 * <P>
 * The <i>subCommands</i> will be called in First In/First Out (FIFO)
 * order.
 *
 * @param {Function} commandClassRef
 * 		A reference to the constructor of the <code>ICommand</code>.
 */
MacroCommand.prototype.addSubCommand = function( commandClassRef )
{
	this.subCommands.push( commandClassRef );
}

/**
 * Execute this <code>MacroCommand</code>'s <i>SubCommands</i>.
 *
 * <P>
 * The <i>subCommands</i> will be called in First In/First Out (FIFO)
 * order.
 *
 * @param {Notification} notification
 * 		The <code>Notification</code> object to be passed to each entry
 * 		of <i>subCommands</i> list.
 * 
 * @final
 */
MacroCommand.prototype.execute = function( notification )
{
	var len/*Number*/ = this.subCommands.length;
	for( var i/*Number*/=0; i<len; i++ )
	{
		var commandClassRef/*Function*/ = this.subCommands[i];
		var commandInstance/*Command*/ = new commandClassRef().initialize();
		commandInstance.execute(notification);
	}
}