/*
 PureMVC standard Javascript port by Frederic Saunier <frederic.saunier@puremvc.org>
 PureMVC - Copyright(c) 2006-2011 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @classDescription
 * A base singleton <code>Facade</code> implementation.
 * 
 * <P>In PureMVC, the <code>Facade</code> class assumes these responsibilities:
 * 
 * <UL>
 * 
 * <LI>Initializing the <code>Model</code>, <code>View</code>
 * and <code>Controller</code> singletons.
 * <LI>Providing all the applicable methods of the <code>Model</code>,
 * <code>View</code>, & <code>Controller</code> singletons.
 * <LI>Providing a single point of contact to the application for
 * registering <code>Command</code>s and notifying <code>Observer</code>s.
 * 
 * <P>This <code>Facade</code> implementation is a singleton
 * and cannot be instantiated directly, but instead calls the static singleton
 * factory method <code>Facade.getInstance()</code>.
 * 
 * @see org.puremvc.js.standard.core.Controller Controller
 * @see org.puremvc.js.standard.core.Model Model
 * @see org.puremvc.js.standard.core.View View
 * @see org.puremvc.js.standard.patterns.observer.Notification Notification
 * @see org.puremvc.js.standard.patterns.mediator.Mediator Mediator
 * @see org.puremvc.js.standard.patterns.proxy.Proxy Proxy
 * @see org.puremvc.js.standard.patterns.command.SimpleCommand SimpleCommand
 * @see org.puremvc.js.standard.patterns.command.MacroCommand MacroCommand
 *
 * @constructor
 */
function Facade(){}

/**
 * Initialize a <code>Facade</code> instance.
 * 
 * @return {Facade}
 * 		This <code>Facade</code> instance.
 */
Facade.prototype.initialize = function()
{
	this.initializeFacade();
	
	return this;
}

/**
 * The <code>View</code> singleton.
 *
 * @type {View}
 * @private
 */
Facade.prototype.view = null;

/**
 * The <code>Model</code> singleton.
 *
 * @type {Model}
 * @private
 */
Facade.prototype.model = null;

/**
 * The <code>Controller</code> singleton.
 *
 * @type {Controller}
 * @private
 */
Facade.prototype.controller = null;

/**
 * @private
 * Called automatically by the constructor.
 * init the Singleton <code>Facade</code> instance.
 *
 * <P>
 * Override in your subclass to do any subclass specific
 * initializations. Be sure to extend the <code>Facade</code> with the
 * methods and properties on your implementation and call
 * <code>Facade.initializeFacade()</code>.
 */
Facade.prototype.initializeFacade = function()
{
	this.initializeModel();
	this.initializeController();
	this.initializeView();
}

/**
 * @private
 * init the <code>Model</code>.
 *
 * <P>
 * Called by the <code>initializeFacade</code> method. Override this
 * method in your subclass of <code>Facade</code> if one or both of the
 * following are true:
 * 
 * <UL>
 * <LI>You wish to init a different <code>IModel</code>.
 * <LI>You have <code>Proxy</code>s to register with the
 * <code>Model</code> that do not retrieve a reference to the
 * <code>Facade</code> at construction time.
 *
 * <P>
 * Note: This method is <i>rarely</i> overridden; in practice you are
 * more likely to use a <code>Command</code> to create and register
 * <code>Proxy</code>s with the <code>Model</code>, since
 * <code>Proxy</code>s with mutable data will likely need to send 
 * <code>Notification</code>s and thus will likely want to fetch a
 * reference to the <code>Facade</code> during their construction.
 */
Facade.prototype.initializeModel = function()
{
	if( !this.model )
		this.model = Model.getInstance();
}

/**
 * @private
 * init the <code>Controller</code>.
 *
 * <P>
 * Called by the <code>initializeFacade</code> method.
 * Override this method in JSON Object <code>Facade</code>
 * definition if one or both of the following are true:
 * 
 * <UL>
 * <LI>You wish to init a different <code>Controller</code>.
 * <LI>You have <code>Command</code>s to register with the
 * <code>Controller</code> at startup.</code>.
 */
Facade.prototype.initializeController = function()
{
	if( !this.controller )
		this.controller = Controller.getInstance();
}

/**
 * @private
 * Initialize the <code>View</code>.
 *
 * <P>
 * Called by the <code>initializeFacade</code> method.
 * Override this method in your subclass of <code>Facade</code>
 * if one or both of the following are true:
 * 
 * <UL>
 * <LI>You wish to initialize a different <code>IView</code>.</LI>
 * <LI>You have <code>Observer</code>s to register with the
 * <code>View</code></LI>
 * </UL>
 * 
 * <P>
 * Note: This method is <i>rarely</i> overridden; in practice you are
 * more likely to use a <code>Command</code> to create and register
 * <code>Mediator</code>s with the <code>View</code>, since
 * <code>IMediator</code> instances will need to send
 * <code>INotification</code>s and thus will likely want to fetch a
 * reference to the <code>Facade</code> during their construction.
 */
Facade.prototype.initializeView = function()
{
	if( !this.view )
		this.view = View.getInstance();
}

/**
 * Register a <code>Command</code> with the <code>Controller</code> by
 * <code>Notification</code> name.
 *
 * @param {String} notificationName
 * 		The name of the <code>Notification</code> to associate the
 * 		<code>Command</code> with.
 *
 * @param {Class} commandClassRef
 * 		A reference to the Class of the <code>Command</code>.
 */
Facade.prototype.registerCommand = function( notificationName/*String*/, commandClassRef/*Class*/ )
{
	this.controller.registerCommand( notificationName, commandClassRef );
}

/**
 * Remove a previously registered <code>Command</code> to
 * <code>Notification</code> mapping from the <code>Controller</code>.
 *
 * @param {String} notificationName
 * 		The name of the <code>Notification</code> to remove the
 * 		<code>Command</code> mapping for.
 */
Facade.prototype.removeCommand = function(notificationName/*String*/ )
{
	this.controller.removeCommand(notificationName);
}

/**
 * Check if a <code>Command</code> is registered for a given
 * <code>Notification</code>.
 *
 * @param {String} notificationName
 * 		The name of the <code>Notification</code> to verify for the
 * 		existence of a <code>Command</code> mapping for.
 *
 * @return {Boolean}
 * 		A <code>Command</code> is currently registered for the given
 * 		<i>notificationName</i>.
 */
Facade.prototype.hasCommand = function( notificationName/*String*/ )
{
	return this.controller.hasCommand(notificationName);
},

/**
 * Register a <code>Proxy</code> with the <code>Model</code> by name.
 *
 * @param proxy {Proxy}
 * 		The <code>Proxy</code> instance to be registered with the
 * 		<code>Model</code>.
 */
Facade.prototype.registerProxy = function( proxy/*Proxy*/ )
{
	this.model.registerProxy(proxy);
}

/**
 * Retrieve a <code>Proxy</code> from the <code>Model</code> by name.
 *
 * @param {String} proxyName
 * 		The name of the <code>Proxy</code> to be retrieved.
 *
 * @return {Proxy}
 * 		The <code>Proxy</code> instance previously registered with the
 * 		given <i>proxyName</i>.
 */
Facade.prototype.retrieveProxy = function( proxyName/*String*/ )
{
	return this.model.retrieveProxy(proxyName);
}

/**
 * Remove an <code>Proxy</code> from the <code>Model</code> by name.
 *
 * @param {String} proxyName
 * 		The <code>Proxy</code> to remove from the <code>Model</code>.
 *
 * @return {Proxy}
 * 		The <code>Proxy</code> that was removed from the
 * 		<code>Model</code>.
 */
Facade.prototype.removeProxy = function(proxyName /*String*/)
{
	return this.model.removeProxy(proxyName);
}

/**
 * Check if a <code>Proxy</code> is registered.
 *
 * @param {String} proxyName
 * 		The <code>Proxy</code> to verify the existence of a 
 * 		registration with the <code>Model</code>.
 *
 * @return {Boolean}
 * 		A <code>Proxy</code> is currently registered with the given
 * 		<i>proxyName</i>.
 */
Facade.prototype.hasProxy = function(proxyName /*String*/)
{
	return this.model.hasProxy(proxyName);
}

/**
 * Register a <code>Mediator</code> with the <code>View</code>.
 *
 * @param {Mediator} mediator
 * 		A reference to the <code>Mediator</code>.
 */
Facade.prototype.registerMediator = function( mediator/*Mediator*/ )
{
	this.view.registerMediator(mediator);
}

/**
 * Retrieve an <code>IMediator</code> from the <code>View</code>.
 *
 * @param {String} mediatorName
 * 		The name of the registered <code>Mediator</code> to retrieve.
 *
 * @return {Mediator}
 * 		The <code>Mediator</code> previously registered with the given
 * 		<i>mediatorName</i>.
 */
Facade.prototype.retrieveMediator = function( mediatorName/*String*/ )
{
	return this.view.retrieveMediator(mediatorName);
}

/**
 * Remove an <code>Mediator</code> from the <code>View</code>.
 *
 * @param {String} mediatorName
 * 		The name of the <code>Mediator</code> to be removed.
 *
 * @return {Mediator}
 * 		The <code>Mediator</code> that was removed from the 
 * 		<code>View</code>.
 */
Facade.prototype.removeMediator = function( mediatorName/*String*/ )
{
	return this.view.removeMediator(mediatorName);
}

/**
 * Check if a <code>Mediator</code> is registered or not.
 *
 * @param {String} mediatorName
 * 		The name of the <code>Mediator</code> to verify the existence
 * 		of a registration for.
 *
 * @return {Boolean}
 * 		A <code>Mediator</code> is registered with the given
 * 		<i>mediatorName</i>.
 */
Facade.prototype.hasMediator = function( mediatorName/*String*/ )
{
	return this.view.hasMediator(mediatorName);
}

/**
 * Create and send a <code>Notification</code>.
 *
 * <P>Keeps us from having to construct new notification instances in
 * our implementation code.
 *
 * @param {String} notificationName
 * 		The name of the notification to send.
 *
 * @param {Object} body
 * 		The body of the notification to send.
 *
 * @param {String} type
 * 		The type of the notification to send.
 */
Facade.prototype.sendNotification = function( notificationName/*String*/, body/*Object*/, type/*String*/ )
{
	this.notifyObservers( new Notification().initialize( notificationName, body, type ) );
}

/**
 * Notify <code>Observer</code>s.
 *
 * <P>
 * This method is left <strong>public</strong> mostly for backward
 * compatibility, and to allow you to send custom notification classes
 * using the <code>Facade</code>.
 *
 *<P>
 * Usually you should just call <i>sendNotification</i> and pass the
 * parameters, never having to construct the <code>Notification</code>
 * yourself.
 *
 * @param {Notification} notification
 * 		The <code>Notification</code> to have the <code>View</code>
 * 		notify <code>Observers</code> of.
 */
Facade.prototype.notifyObservers = function( notification )
{
	this.view.notifyObservers(notification);
}

/**
 * @type {Facade}
 * @private
 */
Facade.instance = null;

/**
 * Retrieve the singleton instance of the <code>Facade</code>.
 * 
 * @return {Facade}
 * 		The singleton instance of the <code>Facade</code>.
 */
Facade.getInstance = function()
{
	if( !Facade.instance )
		Facade.instance = new Facade().initialize();

	return Facade.instance;
}