/*
 PureMVC standard Javascript port by Frederic Saunier <frederic.saunier@puremvc.org>
 PureMVC - Copyright(c) 2006-2011 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @classDescription
 * <P>A base <code>Mediator</code> implementation.
 * 
 * <P>
 * Typically, a <code>Mediator</code> will be written to serve one specific
 * control or group controls and so, will not have a need to be dynamically
 * named.
 * 
 * @see org.puremvc.js.standard.patterns.observer.Notification Notification
 * @extends org.puremvc.js.standard.patterns.observer.Notifier Notifier
 * 
 * @constructor
 */
function Mediator(){}

/**
 * @extends org.puremvc.js.standard.patterns.observer.Notifier Notifier
 */
Mediator.prototype = new Notifier();

/**
 * @override
 *
 * Initialize a <code>Mediator</code> instance.
 *
 * @param {String} mediatorName
 * 		The name of the <code>Mediator</code>.
 *
 * @param {Object} viewComponent
 * 		The <code>Mediator</code>'s view component.
 * 
 * @return {Mediator}
 * 		This <code>Mediator</code> instance.
 */
Mediator.prototype.initialize = function( mediatorName/*String*/, viewComponent/*Object*/ )
{
	Notifier.prototype.initialize.call(this);

	this.mediatorName = (mediatorName != null) ? mediatorName : Mediator.NAME;
	this.viewComponent = viewComponent;
	
	return this;
}

/**
 * The name of the <code>Mediator</code>.
 * 
 * @type {String}
 * @private
 */
Mediator.prototype.mediatorName = null;

/**
 * The <code>Mediator</code>'s view component.
 * 
 * @type {Object}
 * @private
 */
Mediator.prototype.viewComponent = null;


/**
 * List the <code>Notification</code> names this
 * <code>Mediator</code> is interested in being notified of.
 *
 * @return {Array}
 * 		The list of notifications names in which is interested the
 * 		<code>Mediator</code>.
 */
Mediator.prototype.listNotificationInterests = function()
{
	return [];
}

/**
 * Get the name of the <code>Mediator</code>.
 *
 * @return {String}
 * 		The <code>Mediator</code> name.
 */
Mediator.prototype.getMediatorName = function()
{
	return this.mediatorName;
}

/**
 * Get the <code>Mediator</code>'s view component.
 *
 * @return {Object}
 * 		The view component.
 */
Mediator.prototype.getViewComponent = function()
{
	return this.viewComponent;
}

/**
 * Set the <code>Mediator</code>'s view component.
 *
 * @param {Object} viewComponent
 * 		The view component.
 */
Mediator.prototype.setViewComponent = function( viewComponent/*Object*/ )
{
	this.viewComponent = viewComponent;
}

/**
 * Handle <code>INotification</code>s.
 *
 * <P>
 * Typically this will be handled in a switch statement,
 * with one 'case' entry per <code>Notification</code>
 * the <code>Mediator</code> is interested in.
 *
 * @param {Notification} notification
 * 		The notification instance to be handled.
 */
Mediator.prototype.handleNotification = function( notification ){}

/**
 * Called by the View when the Mediator is registered.
 * This method is usually overridden as needed by the subclass.
 */
Mediator.prototype.onRegister = function(){}

/**
 * Called by the View when the Mediator is removed.
 * This method is usually overridden as needed by the subclass.
 */
Mediator.prototype.onRemove = function(){}

/**
 * Default name of the <code>Mediator</code>.
 * 
 * @type {String}
 * @constant
 */
Mediator.NAME = "Mediator";