/*
 PureMVC standard Javascript port by Frederic Saunier <frederic.saunier@puremvc.org>
 PureMVC - Copyright(c) 2006-2011 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @classDescription
 * A base <code>Observer</code> implementation.
 *
 * <P>
 * An <code>Observer</code> is an object that encapsulates information
 * about an interested object with a method that should
 * be called when a particular <code>Notification</code> is broadcast.
 *
 * <P>
 * In PureMVC, the <code>Observer</code> class assumes these responsibilities:
 * <UL>
 * <LI>Encapsulate the notification (callback) method of the interested object.
 * <LI>Encapsulate the notification context (this) of the interested object.
 * <LI>Provide methods for setting the notification method and context.
 * <LI>Provide a method for notifying the interested object.
 *
 * @see org.puremvc.js.standard.core.View View
 * @see org.puremvc.js.standard.patterns.observer.Notification Notification
 * 
 * @constructor
 */
function Observer(){}

/**
 * Initialize an <code>Observer</code> instance.
 *
 * @param {Function} notifyMethod
 * 		The notification method of the interested object.
 * 
 * @param {Object} notifyContext
 * 		The notification context of the interested object.
 * 
 * @return {Observer}
 * 		This <code>Observer</code> instance.
 */
Observer.prototype.initialize = function( notifyMethod/*Function*/, notifyContext/*Object*/ )
{
	this.setNotifyMethod( notifyMethod );
	this.setNotifyContext( notifyContext );

	return this;
}

/**
 * The notification method of the interested object.
 * 
 * @type {Function}
 * @private
 */
Observer.prototype.notify = null;

/**
 * The notification context of the interested object.
 * 
 * @type {Object}
 * @private
 */
Observer.prototype.context = null;

/**
 * Get the notification method.
 *
 * @return {Function}
 * 		The notification (callback) method of the interested object.
 */
Observer.prototype.getNotifyMethod = function()
{
	return this.notify;
}

/**
 * Set the notification method.
 *
 * <P>The notification method should take one parameter of type
 * <code>Notification</code>.</P>
 *
 * @param {Function} notifyMethod
 * 		The notification (callback) method of the interested object.
 */
Observer.prototype.setNotifyMethod = function( notifyMethod/*Function*/ )
{
	this.notify = notifyMethod;
}

/**
 * Get the notification context.
 *
 * @return {Object}
 * 		The notification context (<code>this</code>) of the interested
 * 		object.
 */
Observer.prototype.getNotifyContext = function()
{
	return this.context;
}

/**
 * Set the notification context.
 *
 * @param {Object} notifyContext
 * 		The notification context (this) of the interested object.
 */
Observer.prototype.setNotifyContext = function( notifyContext/*Object */ )
{
	this.context = notifyContext;
}

/**
 * Notify the interested object.
 *
 * @param {Notification} notification
 * 		The <code>Notification</code> to pass to the interested object's
 * 		notification method.
 */
Observer.prototype.notifyObserver = function( notification )
{
	this.getNotifyMethod().call( this.getNotifyContext(), notification );
}

/**
 * Compare an object to the notification context.
 *
 * @param {Object} object
 * 		The object to compare.
 *
 * @return {Boolean}
 * 		The object and the notification context are the same.
 */
Observer.prototype.compareNotifyContext = function( object/*Object*/ )
{
	return object === this.getNotifyContext();
}