/*
 PureMVC standard Javascript port by Frederic Saunier <frederic.saunier@puremvc.org>
 PureMVC - Copyright(c) 2006-2011 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @classDescription
 * A base <code>Proxy</code> implementation.
 * 
 * <P>In PureMVC, <code>Proxy</code> classes are used to manage parts of the
 * application's data model.
 *
 * <P>A <code>Proxy</code> might simply manage a reference to a local data object,
 * in which case interacting with it might involve setting and
 * getting of its data in synchronous fashion.
 *
 * <P><code>Proxy</code> classes are also used to encapsulate the application's
 * interaction with remote services to store or retrieve data, in which case,
 * we adopt an asynchronous idiom; setting data (or calling a method) on the
 * <code>Proxy</code> and listening for a <code>Notification</code> to be sent
 * when the <code>Proxy</code> has retrieved the data from the service.
 *
 * @see org.puremvc.js.standard.core.Model Model
 *
 * @extends org.puremvc.js.standard.patterns.observer.Notifier Notifier
 * 
 * @constructor
 */
function Proxy(){}

/**
 * @extends org.puremvc.js.standard.patterns.observer.Notifier Notifier
 */
Proxy.prototype = new Notifier();

/**
 * @override
 *
 * Initialize a <code>Proxy</code> instance.
 *
 * @param {String} proxyName
 * 		The name of the <code>Proxy</code>.
 *
 * @param {Object} data
 * 		An initial data object to be held by the <code>Proxy</code>.
 * 
 * @return {Proxy}
 * 		This <code>Proxy</code> instance.
 */
Proxy.prototype.initialize = function( proxyName/*String*/, data/*Object*/ )
{
	Notifier.prototype.initialize.call(this);

	this.proxyName = (proxyName != null) ? proxyName : Proxy.NAME;
	this.data = data;
	
	return this;
}

/**
 * The data object controlled by the <code>Proxy</code>.
 *
 * @type {Object}
 * @private
 */
Proxy.prototype.data = null;

/**
 * The name of the <code>Proxy</code>.
 * 
 * @type {String}
 * @private
 */
Proxy.prototype.proxyName = null;

/**
 * Gets the proxyName.
 *
 * @return {String}
 * 		The name of the proxy.
 */
Proxy.prototype.getProxyName = function()
{
	return this.proxyName;
}

/**
 * Sets the data object.
 *
 * @param {Object} data
 * 		The data to set.
 */
Proxy.prototype.setData = function( data/*Object*/ )
{
	this.data = data;
}

/**
 * Gets the data.
 *
 * @return {Object}
 * 		The data held in the <code>Proxy.
 */
Proxy.prototype.getData = function()
{
	return this.data;
}

/**
 * Called by the Model when the <code>Proxy</code> is registered.
 * This method is usually overridden as needed by the subclass.
 */
Proxy.prototype.onRegister = function(){}

/**
 * Called by the Model when the <code>Proxy</code> is removed.
 * This method is usually overridden as needed by the subclass.
 */
Proxy.prototype.onRemove = function(){}

/**
 * The default name of the <code>Proxy</code>
 * 
 * @type {String}
 * @constant
 */
Proxy.NAME = "Proxy";